@@ -1,10 +1,10 @@
 #![recursion_limit="256"]
 
+use crate::args::{Args, SubCommand};
 use crate::rebuild::Context;
 use env_logger::Env;
+use in_toto::crypto::PrivateKey;
 use structopt::StructOpt;
-use structopt::clap::AppSettings;
-use rebuilderd_common::Distro;
 use rebuilderd_common::api::*;
 use rebuilderd_common::auth::find_auth_cookie;
 use rebuilderd_common::config::*;
@@ -12,11 +12,12 @@ use rebuilderd_common::errors::*;
 use rebuilderd_common::errors::{Context as _};
 use std::fs;
 use std::io::{self, Write};
-use std::path::{Path, PathBuf};
+use std::path::Path;
 use std::time::Duration;
 use tokio::select;
 use tokio::time;
 
+pub mod args;
 pub mod auth;
 pub mod config;
 pub mod diffoscope;
@@ -25,61 +26,17 @@ pub mod proc;
 pub mod rebuild;
 pub mod setup;
 
-#[derive(Debug, StructOpt)]
-#[structopt(global_settings = &[AppSettings::ColoredHelp])]
-struct Args {
-    #[structopt(subcommand)]
-    pub subcommand: SubCommand,
-    #[structopt(short, long)]
-    pub name: Option<String>,
-    #[structopt(short, long)]
-    pub config: Option<PathBuf>,
-}
-
-#[derive(Debug, StructOpt)]
-enum SubCommand {
-    /// Rebuild an individual package
-    Build(Build),
-    /// Connect to a central rebuilderd daemon for work
-    Connect(Connect),
-    /// Invoke diffoscope similar to how a rebuilder would invoke it
-    Diffoscope(Diffoscope),
-}
-
-#[derive(Debug, StructOpt)]
-struct Build {
-    pub distro: Distro,
-    pub input: String,
-    /// Use a specific rebuilder script instead of the default
-    #[structopt(long)]
-    pub script_location: Option<PathBuf>,
-    /// Use diffoscope to generate a diff
-    #[structopt(long)]
-    pub gen_diffoscope: bool,
-}
-
-#[derive(Debug, StructOpt)]
-struct Connect {
-    pub endpoint: Option<String>,
-}
-
-#[derive(Debug, StructOpt)]
-struct Diffoscope {
-    pub a: PathBuf,
-    pub b: PathBuf,
-}
-
-async fn spawn_rebuilder_script_with_heartbeat<'a>(client: &Client, distro: &Distro, item: &QueueItem, config: &config::ConfigFile) -> Result<Rebuild> {
-    let input = item.package.url.to_string();
-
+async fn spawn_rebuilder_script_with_heartbeat<'a>(client: &Client, privkey: &PrivateKey, backend: config::Backend, item: &QueueItem, config: &config::ConfigFile) -> Result<Rebuild> {
     let ctx = Context {
-        distro: &distro,
-        script_location: None,
+        artifact_url: item.package.artifact_url.clone(),
+        input_url: item.package.input_url.clone(),
+        backend,
         build: config.build.clone(),
         diffoscope: config.diffoscope.clone(),
+        privkey,
     };
 
-    let mut rebuild = Box::pin(rebuild::rebuild(&ctx, &input));
+    let mut rebuild = Box::pin(rebuild::rebuild(&ctx));
     loop {
         select! {
             res = &mut rebuild => {
@@ -94,17 +51,24 @@ async fn spawn_rebuilder_script_with_heartbeat<'a>(client: &Client, distro: &Dis
     }
 }
 
-async fn rebuild(client: &Client, config: &config::ConfigFile) -> Result<()> {
+async fn rebuild(client: &Client, privkey: &PrivateKey, config: &config::ConfigFile) -> Result<()> {
     info!("Requesting work from rebuilderd...");
-    match client.pop_queue(&WorkQuery {}).await? {
+    let supported_backends = config.backends.keys().map(String::from).collect::<Vec<_>>();
+    match client.pop_queue(&WorkQuery {
+        supported_backends,
+    }).await? {
         JobAssignment::Nothing => {
             info!("No pending tasks, sleeping for {}s...", IDLE_DELAY);
             time::sleep(Duration::from_secs(IDLE_DELAY)).await;
         },
         JobAssignment::Rebuild(rb) => {
             info!("Starting rebuild of {:?} {:?}",  rb.package.name, rb.package.version);
-            let distro = rb.package.distro.parse::<Distro>()?;
-            let rebuild = match spawn_rebuilder_script_with_heartbeat(&client, &distro, &rb, config).await {
+
+            let backend = config.backends.get(&rb.package.distro)
+                .cloned()
+                .ok_or_else(|| anyhow!("No backend for {:?} configured", rb.package.distro))?;
+
+            let rebuild = match spawn_rebuilder_script_with_heartbeat(client, privkey, backend, &rb, config).await {
                 Ok(res) => {
                     if res.status == BuildStatus::Good {
                         info!("Package successfully verified");
@@ -131,9 +95,9 @@ async fn rebuild(client: &Client, config: &config::ConfigFile) -> Result<()> {
     Ok(())
 }
 
-async fn run_worker_loop(client: &Client, config: &config::ConfigFile) -> Result<()> {
+async fn run_worker_loop(client: &Client, privkey: &PrivateKey, config: &config::ConfigFile) -> Result<()> {
     loop {
-        if let Err(err) = rebuild(client, config).await {
+        if let Err(err) = rebuild(client, privkey, config).await {
             error!("Unexpected error, sleeping for {}s: {:#}", API_ERROR_DELAY, err);
             time::sleep(Duration::from_secs(API_ERROR_DELAY)).await;
         }
@@ -153,20 +117,30 @@ async fn run_worker_loop(client: &Client, config: &config::ConfigFile) -> Result
 
 #[tokio::main]
 async fn main() -> Result<()> {
-    env_logger::init_from_env(Env::default()
-        .default_filter_or("info"));
-
     let args = Args::from_args();
-    let config = config::load(args.config.as_deref())
+
+    let logging = match args.verbose {
+        0 => "info",
+        1 => "debug",
+        _ => "trace",
+    };
+
+    env_logger::init_from_env(Env::default()
+        .default_filter_or(logging));
+
+    let config = config::load(&args)
         .context("Failed to load config file")?;
 
     let cookie = find_auth_cookie().ok();
-    debug!("attempt to load auth cookie resulted in: {:?}",cookie);
+    if cookie.is_some() {
+        debug!("Successfully loaded auth cookie");
+    }
 
     if let Some(name) = args.name {
         setup::run(&name)
             .context("Failed to setup worker")?;
     }
+    let profile = auth::load()?;
 
     match args.subcommand {
         SubCommand::Connect(connect) => {
@@ -179,25 +153,36 @@ async fn main() -> Result<()> {
                     .ok_or_else(|| format_err!("No endpoint configured"))?
             };
 
-            let profile = auth::load()?;
-            let client = profile.new_client(system_config, endpoint, config.signup_secret.clone(), cookie);
-            run_worker_loop(&client, &config).await?;
+            let client = profile.new_client(system_config, endpoint, config.signup_secret.clone(), cookie)?;
+            run_worker_loop(&client, &profile.privkey, &config).await?;
         },
+        // this is only really for debugging
         SubCommand::Build(build) => {
-            // this is only really for debugging
-            let mut diffoscope = config::Diffoscope::default();
-            if build.gen_diffoscope {
-                diffoscope.enabled = true;
-            }
+            let backend = if let Some(script_location) = build.script_location {
+                config::Backend {
+                    path: script_location,
+                }
+            } else {
+                config.backends.get(&build.distro)
+                    .cloned()
+                    .ok_or_else(|| anyhow!("No backend configured in config file"))?
+            };
+
+            let diffoscope = config::Diffoscope {
+                enabled: build.gen_diffoscope,
+                ..Default::default()
+            };
 
             let res = rebuild::rebuild(&Context {
-                distro: &build.distro,
-                script_location: build.script_location.as_ref(),
+                artifact_url: build.artifact_url,
+                input_url: build.input_url,
+                backend,
                 build: config::Build::default(),
                 diffoscope,
-            }, &build.input).await?;
+                privkey: &profile.privkey,
+            }).await?;
 
-            debug!("rebuild result object is {:?}", res);
+            trace!("rebuild result object is {:?}", res);
 
             if res.status == BuildStatus::Good {
                 info!("Package verified successfully");
@@ -212,6 +197,10 @@ async fn main() -> Result<()> {
             let output = diffoscope::diffoscope(&diffoscope.a, &diffoscope.b, &config.diffoscope).await?;
             print!("{}", output);
         },
+        SubCommand::CheckConfig => {
+            let json = serde_json::to_string_pretty(&config)?;
+            println!("{}", json);
+        },
     }
 
     Ok(())
